use anyhow::Result;
use btleplug::api::{Central, CentralEvent, Manager as _, Peripheral as _, ScanFilter, WriteType, ValueNotification};
use btleplug::platform::Manager;
use futures::StreamExt;
use std::time::Duration;
use tokio::time::timeout;
use uuid::Uuid;

#[tokio::main]
async fn main() -> Result<()> {
    let mut peripheral = None;

    let manager = Manager::new().await?;

    // get the first bluetooth adapter
    let adapters = manager.adapters().await?;
    let central = adapters
        .into_iter()
        .next()
        .expect("No Bluetooth adapters found");
    let desired_name = "VKA";

    let mut events = central.events().await?;

    // start scanning for devices
    central.start_scan(ScanFilter::default()).await?;

    const TIMEOUT_DURATION: Duration = Duration::from_secs(10);
    while let Some(event) = timeout(TIMEOUT_DURATION, events.next()).await? {
        if let CentralEvent::DeviceDiscovered(addr) = event {
            let some_peripheral = central.peripheral(&addr).await?;
            if let Some(properties) = some_peripheral.properties().await? {
                if let Some(name) = properties.local_name {
                    if name == desired_name {
                        println!("Peripheral found");
                        peripheral = Some(some_peripheral);
                        break;
                    }
                }
            }
        }
    }

    let peripheral = peripheral.expect("Peripheral not found");

    // connect to the device
    peripheral.connect().await?;
    println!("VKA connected");

    // Discover services and characteristics
    peripheral.discover_services().await?;

    // Get a list of the peripheral's characteristics.
    let characteristics = peripheral.characteristics();

    // Filter the characteristics to find the one you're interested in.
    let char_uuid = Uuid::parse_str("DA2E7828-FBCE-4E01-AE9E-261174997C48")?;
    let desired_char = characteristics
        .into_iter()
        .find(|c| c.uuid == char_uuid)
        .expect("SMP characteristic not found");

    // Subscribe to notifications from the characteristic.
    peripheral.subscribe(&desired_char).await?;

    // Write bytes to the characteristic.
    let bytes_to_write = vec![0x00, 0x00, 0x00, 0x01, 0x00, 0x01, 0x80, 0x00, 0xa0];
    peripheral
        .write(&desired_char, &bytes_to_write, WriteType::WithoutResponse)
        .await?;

    let mut notification_stream = peripheral.notifications().await?;
    loop {
        match notification_stream.next().await {
            Some(ValueNotification { uuid, value, .. }) if uuid == desired_char.uuid => {
                println!("Received data: {:?}", value);
                break;
            }
            _ => {}
        }
    }

    Ok(())
}
